﻿/*-----------------------------------------
%
% FD
%
% Algorithm ---------------------------
% X_1 = A * S_1
% X_2 = A * S_2
% X_3 = A * S_3
%...
% X_N = A * S_N
% Sit = [S_1, S_2, S_3, ..., S_N]'
% Sot = [X_1, X_2, X_3, ..., X_N]'
% hatA = inv(Sit' * Sit) * Sit' * Sot
%



#  Arguments ---------------------------
#  ARG01(src)        : src number [Integer]
#  ARG02(inFile)     : Input wav file name [String]
#  ARG03(outFile)    : Output flt filename prefix. The name will be [outDir outFile '_ch*.flt']  [String]
#  ARG04(s)          : start point for Transfer Function(TF) calculation in sample [Integer]
#  ARG05(e)          : end point for TF calculation in sample [Integer]
#  ARG06(nch)        :  Channel Total number  [Integer]
#  ARG07(fltlen)     : Number of samples output as TF [Integer]
#  ARG08(nfft)       : Number of FFT points [Integer]
#  ARG09(hop)        : Shift length for TF calculation [Integer]
#  ARG10(headmargin) : Head margin of impulse responces in samples [Integer(Default:0)]
#  ARG11(eneble_wn)  : Enables adding white noise to escape unstable zero signal [Bool]
#  ARG12(wn_level)   : Level of white noise. Set enable_wn=true if you want to use this. [Double]
#  ARG13(enable_normalize) : Enable normalizing output TF


------------------------------------------------------------------------------------*/


#include <unistd.h>
#include "harktool.h"


#define ORG_MTSP (16384 * 3 +1 )


extern struct GLOBAL  g;




typedef struct {
    float re; /// real part
    float im; /// imaginary part
} mtsp_Complex;



mtsp_Complex*  mtsp_spec;
mtsp_Complex*  mtsp_complexWave;
mtsp_Complex** mtsp_Si;
mtsp_Complex   mtsp_Sit[33][1];
mtsp_Complex   mtsp_Sit_[1][33];
mtsp_Complex** mtsp_So;
mtsp_Complex   mtsp_Sot[33][1];
mtsp_Complex** mtsp_Ro;
mtsp_Complex   mtsp_buf_hc;
mtsp_Complex*  mtsp_A;
float*         mtsp_wave;
float**        mtsp_YY;




//プロトタイプ宣言
void CalcHammingWindow(float *w, int length);
void mtsp_fft(int nfft);
void mtsp_ifft(int nfft);
int harktool_readWaveHeader(FILE *fp ,
                            unsigned short *channelnum  ,
                            unsigned long  *samplingrate ,
                            unsigned short *bit_per_sample,
                            unsigned long  *datasize);
mtsp_Complex** harktool_newMtsp_ComplexTwoDim(int size_x , int size_y);
void harktool_freeMtsp_ComplexTwoDim(mtsp_Complex** matrix , int size_x);







int harktool_mouthTspFD(int src              ,  // src number
                        char *inFile           , // Input wav file name [String]
                        char *outFile          , // Output flt filename prefix. The name will be [outDir outFile '_ch*.flt']  [String]
                        int s                , // start point for Transfer Function(TF) calculation in sample [Integer]
                        int e                , // end point for TF calculation in sample [Integer]
                        int mch              , // Channel index of the voice input. The first channel is 1ch not 0ch. [Integer]
                        int nch              , // ChannelSelector for micarray channels.(MAX channnel)
                        int fltlen           , // Number of samples output as TF [Integer]
                        int nfft             , // Number of FFT points [Integer]
                        int hop              , // Shift length for TF calculation [Integer]
                        int headmargin       , // Head margin of impulse responces in samples [Integer(Default:0)]
                        //startid          , // start id of flt files like _ch0.flt if 0 [Integer(Default:1)]
                        int enable_wn        , // Enables adding white noise to escape unstable zero signal [Bool]
                        double wn_level         , // Level of white noise. Set enable_wn=true if you want to use this. [Double]
                        int enable_normalize   // Enable normalizing output TF
                       )
{



#if 0 //hikisuu
//    char*       inFile = FNAMES(i).name;
//    char*       outFile = [FNAMES(i).name(1:end-4),'-FD'];
//    int         s = nfft * 1;	// e - s <= 16384
//    int         e = nfft * 4;	// 65536
//    int         mch = 1; //
//    int         nch = max channel; // max = 8 ch
//    int         fltlen = 4096;
//    int         nfft = nfft; // nfft <= 16384
//    int         hop = 1000;
//    int         headmargin = 128;
//    int         enable_wn = false;
//    double      wn_level = 0.0001;
//    int         enable_normalize = false;
#endif







    char	file_w[256];
    FILE	*fp_r, *fp_w;
    char	tsp_fname[PATH_LEN];
    char	*ptr;
    char	reg_c[3];
//    int		bit = (int)bit_per_sample;	// 16 or 24 bit
    int		r_num;
    int		shift_num;
    int		ret;
    int		val;
    int		i,j,k,n;
    int		input_org_size;
    int		output_org_size;
    int		reg_i[1];
    float	int_max;
    float	window[nfft];
    float	input_org[ORG_MTSP];
    float	output_org[ORG_MTSP][8];
    float	reg_f[513];
    float	reg_out_f[nch][nfft/2+1];
    float	max_f;

    int     mic;
    int     fft;
    float   real,imag;

    unsigned short channelnum;			//チャンネル数
    unsigned long  samplingrate;		//サンプリング周波数
    unsigned short bit_per_sample;		//量子化ビット数
    unsigned long  datasize;			//波形データのバイト数



    memset(input_org,'\0', ORG_MTSP*sizeof(float));
    memset(output_org,'\0',ORG_MTSP*8*sizeof(float));


    //領域設定
    mtsp_spec         = (mtsp_Complex*)malloc((nfft+1) * sizeof(mtsp_Complex));
    mtsp_complexWave  = (mtsp_Complex*)malloc((nfft+1) * sizeof(mtsp_Complex));
    mtsp_Si           =  harktool_newMtsp_ComplexTwoDim(33 , (nfft/2+1));
    mtsp_So           =  harktool_newMtsp_ComplexTwoDim(33 , (nfft/2+1));
    mtsp_Ro           =  harktool_newMtsp_ComplexTwoDim((nfft/2+1) , nch);
    mtsp_A            = (mtsp_Complex*)malloc((nfft/2+1) * sizeof(mtsp_Complex));
    mtsp_wave         = (float*)malloc((nfft * 2) * sizeof(float));
    mtsp_YY           =   harktool_newFloatTwoDim(nfft , nch);


    s--          ;
    e--          ;
    headmargin-- ;
    nch--        ;
    mch--        ;





    /*--------------------------------------------------*/
    // Input file
    /*--------------------------------------------------*/
    memset(tsp_fname  , '\0' , sizeof(tsp_fname));

    sprintf(tsp_fname , "%s" ,  inFile);


    if ((fp_r = fopen(tsp_fname, "rb")) == NULL) {
        harkio_Log_printf(harkio_Log_Error , "file open error");
        harktool_Exit(EXIT_FAILURE);
    }


    ret = harktool_readWaveHeader(fp_r ,
                                  &channelnum    , //チャンネル数
                                  &samplingrate  , //サンプリング周波数
                                  &bit_per_sample, //量子化ビット数
                                  &datasize        //波形データのバイト数
                                 );


    if (ret == -1) {
        fclose(fp_r);
        harkio_Log_printf(harkio_Log_Error , "wave file read error");
        harktool_Exit(EXIT_FAILURE);
    }



    if (bit_per_sample == 16) {
        r_num     = 2;
        shift_num = 16;
        int_max   = 32767.f;
    } else {
        r_num     = 3;
        shift_num = 8;
        int_max   = 32768.f * 128.f;
    }




    long poswav = ftell(fp_r);
    /*--------------------------------------------------*/
    /*	mch の s - e の抜き出し	*/
    /*--------------------------------------------------*/
    ret = fseek(fp_r , s * r_num * (nch+1) + poswav, SEEK_SET);
    ptr = (char *)&val; // val:int
    k   = 0;

    for (i = 0; i<=e-s; i++) {
        for (j = 0; j<(nch+1); j++) {
            ret = fread(&reg_c[0], sizeof(char), r_num, fp_r);
            val = 0;

            if (bit_per_sample == 16) {
                ptr[2] = reg_c[0];
                ptr[3] = reg_c[1];
            } else {
                ptr[1] = reg_c[0];
                ptr[2] = reg_c[1];
                ptr[3] = reg_c[2];
            }

            //mch only
            if (j == mch) {
                val = val >> shift_num;
                input_org[k] = (float)val / int_max;
                k++;
            }
        }
    }

    input_org_size = k;


    fclose(fp_r);




    /*--------------------------------------------------*/
    // Input File
    /*--------------------------------------------------*/
    if ((fp_r = fopen(tsp_fname, "rb")) == NULL) {
        harkio_Log_printf(harkio_Log_Error , "file open error");
        harktool_Exit(EXIT_FAILURE);
    }


    /*	全 ch の抜き出し( -1: 不明	)	*/
    ret = fseek(fp_r, (s-headmargin-1) * r_num * (nch+1)+poswav, SEEK_SET);
    ptr = (char *)&val;


    for (i = 0; i<=(e-s); i++) {
        for (j = 0; j<(nch+1); j++) {


            ret = fread(&reg_c[0], sizeof(char), r_num, fp_r);
            val = 0;

            if (bit_per_sample == 16) {
                ptr[2] = reg_c[0];
                ptr[3] = reg_c[1];
            } else {
                ptr[1] = reg_c[0];
                ptr[2] = reg_c[1];
                ptr[3] = reg_c[2];
            }
            val = val >> shift_num;
            output_org[i][j] = (float)val / int_max;
        }
    }

    output_org_size = i;
    fclose(fp_r);



    /*--------------------------------------------------*/
    /*	Input                                           */
    /*--------------------------------------------------*/
    reg_f[0] = (float)(input_org_size - (float)(nfft-hop)) / (float)hop;
    reg_i[0] = (int)reg_f[0];
    reg_i[0] = reg_i[0] - 1;


    CalcHammingWindow(window, nfft);


    for (i = 0; i < nfft; i++) {
        mtsp_spec[i].re = 0.0f;
        mtsp_spec[i].im = 0.0f;

        mtsp_complexWave[i].re = 0.0f;
        mtsp_complexWave[i].im = 0.0f;
    }


    for (i = 0; i <= reg_i[0]; i++) {
        k = 0;
        for (j = (i*hop+1-1) ; j < (i*hop+nfft) ; j++) {
            mtsp_spec[k].re = window[k] * input_org[j];
            k++;
        }

        mtsp_fft(nfft);


        //for (n = 0; n < nfft; n++) {
        //    mtsp_complexWave[n].im = mtsp_complexWave[n].im;// * (-1);
        //}


        for (n = 0; n < nfft/2+1; n++) {
            mtsp_Si[i][n].re = mtsp_complexWave[n].re;
            mtsp_Si[i][n].im = mtsp_complexWave[n].im;
        }
    }







    /*--------------------------------------------------*/
    /*	Output                                          */
    /*--------------------------------------------------*/
    reg_f[0] = (float)(output_org_size - (float)(nfft-hop)) / (float)hop;
    reg_i[0] = (int)reg_f[0];
    reg_i[0] = reg_i[0] - 1;

    for (i = 0; i < (nch+1); i++) {
        for (j = 0; j <= reg_i[0]; j++) {
            for (n = 0; n < nfft; n++) {
                mtsp_spec[n].re = 0.f;
                mtsp_spec[n].im = 0.f;
                mtsp_complexWave[n].re = 0.f;
                mtsp_complexWave[n].im = 0.f;
            }

            k = 0;
            for (n = j*hop+1-1; n < j*hop+nfft; n++) {
                mtsp_spec[k].re = window[k] * output_org[n][i];
                k++;
            }

            mtsp_fft(nfft);


            //for (n = 0; n < nfft; n++) {
            //    mtsp_complexWave[n].im = mtsp_complexWave[n].im;// * (-1);
            //}

            for (n = 0; n < nfft/2+1; n++) {
                mtsp_So[j][n] = mtsp_complexWave[n];
            }

        }// j (reg_i[0] )End ...



        for (j = 0; j < nfft/2+1; j++) {
            for (n = 0; n <= reg_i[0]; n++) {
                mtsp_Sit[n][0]		= mtsp_Si[n][j];
                mtsp_Sit_[0][n].re	= mtsp_Sit[n][0].re;
                mtsp_Sit_[0][n].im	= mtsp_Sit[n][0].im * (-1);
                mtsp_Sot[n][0] 		= mtsp_So[n][j];
            }


            // inv(Sit' * Sit)
            reg_f[0] = 0.0f;
            for (n = 0; n <= reg_i[0]; n++) {
                reg_f[0] = reg_f[0] + (mtsp_Sit_[0][n].re * mtsp_Sit[n][0].re) - (mtsp_Sit_[0][n].im * mtsp_Sit[n][0].im);	// re
            }

            reg_f[0] = 1 / reg_f[0];

            // Sit' * Sot
            reg_f[1] = 0.0f;
            reg_f[2] = 0.0f;

            for (n = 0; n <= reg_i[0]; n++) {
                reg_f[1] = reg_f[1] + (mtsp_Sit_[0][n].re * mtsp_Sot[n][0].re) - (mtsp_Sit_[0][n].im * mtsp_Sot[n][0].im);	// re
                reg_f[2] = reg_f[2] + (mtsp_Sit_[0][n].re * mtsp_Sot[n][0].im) + (mtsp_Sit_[0][n].im * mtsp_Sot[n][0].re);	// im
            }

            //mtsp_Ro[NUM_FFT_MTSP/2+1][_CH_VAL_];
            mtsp_Ro[j][i].re = reg_f[0] * reg_f[1];
            mtsp_Ro[j][i].im = reg_f[0] * reg_f[2];

        }// j (nfft/2+1) End ...
    }// i (nch+1) End...




    for (i = 0; i < (nch+1); i++) {
        for (j = 0; j < nfft/2+1; j++) {
            mtsp_A[j] = mtsp_Ro[j][i];
        }


        mtsp_ifft(nfft);

        for (j = 0; j < nfft; j++) {
            mtsp_YY[j][i] = mtsp_wave[j] / nfft;
        }
    }



    /*--------------------------------------------------*/
    /*	Save Imp Files	 nmic分                         */
    /*--------------------------------------------------*/
    for (i = 0 ; i < (nch+1); i++) {
        sprintf(file_w, "%s_ch%d.flt" ,   outFile , i);

        if ((fp_w = fopen(file_w, "wb")) == NULL) {
            harkio_Log_printf(harkio_Log_Error , "file open error");
            harktool_Exit(EXIT_FAILURE);
        }

        for (j = 0; j < fltlen; j++) { //fltlen=4096
            reg_f[0] = mtsp_YY[j][i];
            fwrite(reg_f , sizeof(float), 1, fp_w);
        }
        fclose(fp_w);
    }


    printf("MTP Debug : nch = %d, fltlen = %d, float size = %d\n", nch, fltlen, (int)sizeof(float));


    exit(0);

    /*--------------------------------------------------*/
    /*	Save TF Files	                                */
    /*--------------------------------------------------*/
    // Loc File
    // Write TF Data : 16448[BYte]= (512/2+1) * 8[ch] * 8[Byte](re & im) Write
    for (i = 0; i < (nch+1) ; i++) {
        for (j = 0; j < (nfft/2+1) ; j++) {
            mtsp_spec[j].re = mtsp_YY[j][i];
            mtsp_spec[j].im = 0.f;
        }

        /*--------------------------------------------------*/
        // インパルス応答をFFTして伝達関数を取得
        /*--------------------------------------------------*/
        mtsp_fft(nfft); //in:mtsp_spec out:mtsp_complexWave

        //float reg_out_f[ch][]
        //reg_out_f[i][0]<- real, reg_out_f[i][1]<- imag
        for (j = 0; j < (nfft/2+1); j++) {
            reg_out_f[i][j*2]   = mtsp_complexWave[j].re;
            reg_out_f[i][j*2+1] = mtsp_complexWave[j].im;// * (-1);
        }
    }


    printf("size=%ld\n",sizeof(g.TFvect));

    for (mic=0 ; mic<(nch+1) ; mic++) {
        for (fft=0 ; fft< (nfft/2+1) ; fft++) {
            real = mtsp_complexWave[fft].re;  ;
            imag = mtsp_complexWave[fft].im * (-1);

            g.TFvect[src][mic][fft] = real  + imag * _Complex_I;

        }
    }





    /*--------------------------------------------------*/
    // Sep File
    /*--------------------------------------------------*/
    // Offset = 1200 Byte
    // Write TF Data : 16448[BYte]= (257) * 8[ch] * 8[Byte](re & im) Write
    max_f = 0.f;
    for (i = 0; i < (nch+1); i++) {
        for (j = 0; j < (nfft/2+1); j++) {
            if (j%2) {
                reg_out_f[i][j] = reg_out_f[i][j] * (-1);
            }

            if (max_f < abs(reg_out_f[i][j])) {
                max_f = abs(reg_out_f[i][j]);
            }
        }
    }

    for (i = 0 ; i < (nch+1) ; i++) {
        for (j = 0 ; j < (nfft/2+1) ; j++) {
            if (max_f != 0.f) {
                reg_out_f[i][j] = reg_out_f[i][j] / max_f;
            }
        }
        //fwrite(reg_out_f[i], sizeof(float), 514, fp_w);
    }



    for (mic=0 ; mic<(nch+1) ; mic++) {
        for (fft=0 ; fft< (nfft/2+1) ; fft++) {
            real = reg_out_f[mic][fft*2]  ;
            imag = reg_out_f[mic][fft*2+1];

            g.SepTFvect[src][mic][fft] = real  + imag * _Complex_I;
        }
    }






    //領域解放
    free(mtsp_spec);
    free(mtsp_complexWave);
    free(mtsp_A);
    free(mtsp_wave);

    harktool_freeMtsp_ComplexTwoDim(mtsp_Si , 33);
    harktool_freeMtsp_ComplexTwoDim(mtsp_So , 33);
    harktool_freeMtsp_ComplexTwoDim(mtsp_Ro  , (nfft/2+1));
    harktool_freeFloatTwoDim(mtsp_YY , nfft);


    return (EXIT_SUCCESS);

}//end









/*-----------------------------------------------------------------------------*
*
*
*
*-----------------------------------------------------------------------------*/
void CalcHammingWindow(float *w, int length)
{
    int n;
    float	fft_window_gain = 1.0f;

    float	c = 2.0f * M_PI / ((float)length - 1);

    for (n = 0; n < length; n++) {
        w[n] = (0.54f - 0.46f * cos(c * n)) * fft_window_gain;
    }
}







/*-----------------------------------------------------------------------------*
* fft
*
*
*-----------------------------------------------------------------------------*/
void mtsp_fft(int nfft)
{

    int i;

    COMPLEX_FLOAT aa[nfft];

    for (i = 0; i < nfft; i++) {
        aa[i] = mtsp_spec[i].re + mtsp_spec[i].im * _Complex_I;
    }

    harktool_FFTImpl_fFFT(aa, aa, nfft);

    for (i = 0; i < nfft; i++) {
        mtsp_complexWave[i].re = crealf(aa[i]);
        mtsp_complexWave[i].im = cimagf(aa[i]);
    }

}//






/*-----------------------------------------------------------------------------*
* ifft
* [nfft/2+1, nfft-1]の要素はゼロ埋めしてIFFT
*
*-----------------------------------------------------------------------------*/
void mtsp_ifft(int nfft)
{

    int i;
    COMPLEX_FLOAT aa[nfft];

    for (i = 0; i < (nfft/2+1); i++) {
        aa[i] = mtsp_A[i].re + mtsp_A[i].im * _Complex_I;
    }
    for (; i < nfft; i++) { // ゼロ埋め
        aa[i] = 0;
    }

    harktool_FFTImpl_fFFTInv(aa, aa, nfft);

    for (i = 0; i < nfft; i++) {
        mtsp_wave[i] = crealf(aa[i]);
    }

}










/*-----------------------------------------------------------------------------*
* Waveファイル処理
*
*
//	unsigned short channelnum;			//チャンネル数
//	unsigned long samplingrate;			//サンプリング周波数
//	unsigned short bit_per_sample;		//量子化ビット数
*
*-----------------------------------------------------------------------------*/
int harktool_readWaveHeader(FILE *fp ,
                            unsigned short *channelnum  ,
                            unsigned long  *samplingrate ,
                            unsigned short *bit_per_sample,
                            unsigned long  *datasize)
{





    unsigned char  header_buf[20];		//フォーマットチャンクのサイズまでのヘッダ情報を取り込む
    unsigned short fmtid;				//fmtのIDを格納する
    unsigned char *buf;					//フォーマットチャンクIDから拡張部分までのデータを取り込む
    unsigned long fmtsize;

    unsigned short tmp_channelnum;		//チャンネル数
    unsigned long  tmp_samplingrate;	//サンプリング周波数
    unsigned short tmp_bit_per_sample;	//量子化ビット数
    unsigned long  tmp_datasize;		//波形データのバイト数





    //フォーマットチャンクサイズまでのヘッダ部分を取り込む
    fread(header_buf, sizeof(unsigned char), 20, fp);

    //ファイルがRIFF形式であるか
    //int strncmp(const char *s1, const char *s2, size_t n);
    if (strncmp((const char*)header_buf, "RIFF", 4) != 0) {
        fprintf(stderr, "Error:  not found RIFF ID.");
        return (-1);
    }

    //ファイルがWAVEファイルであるか
    if (strncmp((const char*)header_buf + 8, "WAVE", 4) != 0) {
        fprintf(stderr, "Error: not found WAVE ID.");
        return (-1);
    }


    //fmt のチェック
    if (strncmp((const char*)header_buf + 12, "fmt ", 4)  != 0) {
        fprintf(stderr, "Error:  not found fmt id.");
        return (-1);
    }



    memcpy(&fmtsize, header_buf + 16, 4);

    printf("fmsize=[%ld] \n", fmtsize);



    if ((buf = (unsigned char *)malloc(sizeof(unsigned char)*fmtsize)) == NULL) {
        fprintf(stderr, "Allocation error\n");
        return (-1);
    }




    //フォーマットIDから拡張部分までのヘッダ部分を取り込む
    fread(buf, 1, fmtsize, fp);

    //LinearPCMファイルならば1が入る
    memcpy(&fmtid, buf, 2);

    if (fmtid!=1) {
        fprintf(stderr, "Error: not LinearPCM.");
        free(buf);
        return (-1);
    }

    memcpy(&tmp_channelnum    , buf + 2  , 2);	    //チャンネル数を取得
    memcpy(&tmp_samplingrate  , buf + 4  , 4);   //サンプリング周波数を取得
    memcpy(&tmp_bit_per_sample, buf + 14 , 2); //量子化ビット数を取得

    //factもしくはdataのIDとサイズを取得8バイト
    fread(buf, sizeof(unsigned char), 8, fp);


    if (strncmp((const char*)buf, "fact", 4) == 0) {
        fread(buf, sizeof(unsigned char), 4, fp);
        fread(buf, sizeof(unsigned char), 8, fp);
    }

    if (strncmp((const char*)buf, "data", 4) != 0) {
        fprintf(stderr, "Error:not found data id.");
        free(buf);
        return (-1);
    }


    /*----------------------------------------------------------------*/
    //波形データのサイズの取得
    /*----------------------------------------------------------------*/
    memcpy(&tmp_datasize , buf + 4, 4);  //波形データのサイズの取得




    *channelnum     = tmp_channelnum     ;
    *samplingrate   = tmp_samplingrate   ;
    *bit_per_sample = tmp_bit_per_sample ;
    *datasize       = tmp_datasize       ;

    free(buf);
    return 0;

}







/*-----------------------------------------------------------------------------*
* 2次元配列を作成する()
*  size_x * size_y
*
*
*-----------------------------------------------------------------------------*/
mtsp_Complex** harktool_newMtsp_ComplexTwoDim(int size_x , int size_y)
{

    int i;
    mtsp_Complex **m;


    /* size_x 分のアドレス格納領域を確保 */
    m = (mtsp_Complex**)malloc(sizeof(mtsp_Complex*) * size_x) ;

    if (m == NULL) {
        harktool_LogE("can not allocated memory");
        harktool_Exit(EXIT_FAILURE);
    }


    /* size_y分 のfloatデータ格納領域を確保 */
    for (i=0 ; i < size_x ; i++) {
        m[i] = (mtsp_Complex*)malloc(sizeof(mtsp_Complex) * size_y);

        if (m[i] ==NULL) {
            harktool_LogE("can not allocated memory");
            harktool_Exit(EXIT_FAILURE);
        }
        memset(m[i] , '\0' , sizeof(mtsp_Complex) * size_y);

    }

    return (m);


}//end




/*-----------------------------------------------------------------------------*
* 確保した2次元配列を解放(float)
*
*
*
*-----------------------------------------------------------------------------*/
void harktool_freeMtsp_ComplexTwoDim(mtsp_Complex** matrix , int size_x)
{

    int i;

    if (matrix == NULL) return;


    /* 確保した配列を解放 */
    for (i=0 ; i < size_x ; i++) {
        if (matrix[i] != NULL) {
            free(matrix[i]);
            matrix[i] = NULL;
        }
    }

    if (matrix != NULL)  free(matrix);
    matrix = NULL;

}






