#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include <libharkio3.h>

#define  PATH_LEN     1024     // file path 最大長




void help(void)
{
    printf("                                                                    \n");
    printf("                                                                    \n");
    printf("[NAME:]                                                             \n");
    printf("  harktoolcli-conv-sm                                               \n");
    printf("                                                                    \n");
    printf("[Description:]                                                      \n");
    printf("  Convert the old file separation matrix                            \n");
    printf("                                                                    \n");
    printf("[Usage :]                                                           \n");
    printf("  harktoolcli-conv-sm [options]                                     \n");
    printf("                                                                    \n");
    printf("[options]                                                           \n");
    printf("  -m : Separation matrix file(Binary)              (-m mat.tff)     \n");
    printf("  -s : Corresponding transfer function (zip file)  (-s sep.zip)     \n");
    printf("  -o : output separation matrix (zip file)         (-o output.zip)  \n");
    printf("                                                                    \n");
    printf("                                                                    \n");
}









int main(int argc, char **argv)
{

    int option;
    char *sepmatpath = NULL;
    char *septfpath= NULL;
    char *outfile= NULL;
    char *whatisthis= "separation matrix";

    harkio_TransferFunction *tf = NULL;


    //argument check
    if (argc <= 1) {
        harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help();
        exit(EXIT_FAILURE);
    }

    // help
    if (strcmp(argv[1] , "--help") == 0) {
        help();
        exit(EXIT_SUCCESS);
    }

    // parse option
    while ((option = getopt(argc, argv, "o:m:s:")) != -1) {
        switch (option) {
        case 'm':
            sepmatpath = (char *)malloc(strlen(optarg) + 1);
            memset(sepmatpath  , '\0' , strlen(optarg) + 1);
            memcpy(sepmatpath , optarg, strlen(optarg));

            break;

        case 's':
            septfpath = (char *)malloc(strlen(optarg) + 1);
            memset(septfpath  , '\0' , strlen(optarg) + 1);
            memcpy(septfpath , optarg, strlen(optarg));
            break;

        case 'o':
            outfile = (char *)malloc(strlen(optarg) + 1);
            memset(outfile  , '\0' , strlen(optarg) + 1);
            memcpy(outfile , optarg, strlen(optarg));
            break;

        case '?':
            help();
            exit(EXIT_FAILURE);
        }
    }

    if (sepmatpath) printf("sepmatpath=%s\n",sepmatpath);

    // result = libharkio3_SeparationMatrix_readlibhakio2( &tf , sepmatpath, septfpath );
    tf = harkio_Compat_SeparationMatrix_readlibharkio2(sepmatpath, septfpath);

    if (tf == NULL) {
        harkio_TransferFunction_delete(&tf);
        harkio_Log_printf(harkio_Log_Error , "harkio_Compat_SeparationMatrix_readlibharkio2 error");
        exit(EXIT_FAILURE);
    }



    //write zip file
    harkio_TransferFunction_write(tf , outfile , whatisthis);


    fprintf(stderr, "SUCCESS\n");
    fprintf(stderr, " output file : %s\n",outfile);

    harkio_TransferFunction_delete(&tf);
    if (sepmatpath !=NULL) free(sepmatpath);
    if (septfpath !=NULL) free(septfpath);
    if (outfile   !=NULL) free(outfile);

    exit(EXIT_SUCCESS);


}//func end


