#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include "libharkio3.h"







void help(void)
{
    printf("                                             \n");
    printf("                                             \n");
    printf("[NAME:]                                      \n");
    printf("  harktoolcli-genneighbors                   \n");
    printf("                                             \n");
    printf("[Description:]                               \n");
    printf("  The command creates an xml file that contains\n");
    printf("  neighbors tag of xml file.                 \n");
    printf("                                             \n");
    printf("[Usage :]                                    \n");
    printf("  harktoolcli-genneighbors [options]         \n");
    printf("                                             \n");
    printf("[options]                                    \n");
    printf("   -i:  Input  filename                      \n");
    printf("   -o:  Output filename                      \n");
    printf("   -a:  Algorithms( 1:NearestNeighbor)       \n");
    printf("   [options for -a 1]                        \n");
    printf("   -n:  Maximum number of neighbors          \n");
    printf("   -t:  Threshold for neighbor decision      \n");
    printf("                                             \n");
    printf("                                             \n");
}






int main(int argc, char **argv)
{

    harkio_XML *xml;
    int  ret;
    FILE *fp;
    int option;
    harkio_Neighbors *nbrs;

    char  input[1024]  ;
    char  output[1024] ;
    int   alg          ;
    int   nearest      ;
    float threshold    ;
    //harkio_Positions *poses;


    memset(input   , '\0' , sizeof(input));
    memset(output  , '\0' , sizeof(output));


    if (argc <= 1) {
        harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help();
        exit(EXIT_FAILURE);
    }

    // help
    if (strcmp(argv[1] , "--help") == 0) {
        help();
        exit(EXIT_SUCCESS);
    }

    // parse option
    while ((option = getopt(argc, argv, "i:o:a:n:t:i:")) != -1) {
        switch (option) {
        case 'i':
            strcpy(input , optarg);
            break;

        case 'o':
            strcpy(output ,optarg);
            break;

        case 'a':
            alg = atoi(optarg);
            break;

        case 'n':
            nearest = atoi(optarg);
            break;

        case 't':
            threshold = atof(optarg);
            break;

        case '?':
            help();
            exit(EXIT_FAILURE);
        }
    }


    if (strcmp(input , output) == 0) {
        harkio_Log_printf(harkio_Log_Error, "file name error. can not same name, input file and output file \n") ;
        exit(EXIT_FAILURE);
    }



    if ((fp = fopen(output , "w")) == NULL) {
        harkio_Log_FileOpenFailed(output);
        exit(EXIT_FAILURE);
    }



    xml =  harkio_XML_newFromXML(input , "");



    nbrs = harkio_Neighbors_new(xml->poses);


    switch (alg) {
    case NearestNeighbor:

        ret = harkio_Neighbors_calcNeighbors(nbrs, NearestNeighbor ,  nearest ,  threshold);

        if (ret == EXIT_FAILURE) {
            harkio_Log_printf(harkio_Log_Error, "harkio_Neighbors_calcNeighbors error\n") ;
            exit(EXIT_FAILURE);
        }

        break;


    default:
        harkio_Log_printf(harkio_Log_Error, "unknown algorithm \n") ;
        exit(EXIT_FAILURE);
    }

    //set neighbors
    xml->nbrs = nbrs;



    // make new XML object and write
    harkio_XML_write(xml, fp);

    harkio_XML_delete(&xml);

    fclose(fp);

    harkio_Log_printf(harkio_Log_Debug, "XML file created");


    exit(EXIT_SUCCESS);


}










