/*-----------------------------------------------------------------------------*
*
*  calctf.c をinclude すること
*
*
*-----------------------------------------------------------------------------*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <libharkio3.h>
#include <math.h>

#include "harktool.h"







float calcZValue(int z_unit , float re , float im);
void calcIFTMatrixData(harkio_Matrix** freqdata, harkio_Matrix** timedata, int num_src, int num_mic, int num_fft);
int writeCoordinateInternal(const char* outfile, int z_axis, harkio_Matrix** matdata, int num_src, int num_mic, int num_x, float x_weight, float y_weight);
// void global_init();


#define DISP_REAL   1
#define DISP_AMP    2
#define DISP_db     3
#define DISP_PHASE  4



//harktool global 構造体
// struct GLOBAL  g;



void help(void)
{
    printf("                                                \n");
    printf("                                                \n");
    printf("[NAME:]                                         \n");
    printf("  harktoolcli-ifftTF                            \n");
    printf("                                                \n");
    printf("[Description:]                                  \n");
    printf("  Coordinate from transfer function             \n");
    printf("                                                \n");
    printf("[Usage :]                                       \n");
    printf("  harktoolcli-ifftTF tf_type tf_filename(zip)   \n");
    printf("                                                \n");
    printf("[options]                                                     \n");
    printf("      -t: Type of Transfer function    ex.(-t LOC)   [string] \n");
    printf("            LOC : Localization transfer function              \n");
    printf("            SEP : Separation   transfer function              \n");
    printf("      -i: input file                     [string]             \n");
    printf("      -o: output file                    [string]             \n");
    printf("                                                              \n");
//    printf("      -z: z_axis type   ( amp: amplitude   db: decibel  phase:phase  [string]  \n");//ifft is amp only
//    printf("      -s: Sampling frequency           (-s 16000)    [integer] \n");
//    printf("      -n: Number of fft                (-n 512)      [integer] \n");
    printf("                                                              \n");

}





int main(int argc, char **argv)
{
    int option;

    char msg[MSG_LEN] = "";

    char infile[PATH_LEN] = "";
    char outfile[PATH_LEN] = "";

    int z_axis = DISP_REAL;
    int tf_type=0;

    harkio_TransferFunction *tf = NULL;



    if (argc <= 1) {
        harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help();
        exit(EXIT_FAILURE);
    }

    // help
    if (strcmp(argv[1] , "--help") == 0) {
        help();
        exit(EXIT_SUCCESS);
    }

    // parse option
    while ((option = getopt(argc, argv, "t:z:o:i:s:n:")) != -1) {
        switch (option) {
        case 't':
            if (strcmp(optarg , "LOC") == 0  ||  strcmp(optarg , "loc") == 0) {
                tf_type=0;
            } else if (strcmp(optarg , "SEP") == 0 || strcmp(optarg , "sep") == 0) {
                tf_type=1;
            } else {
                help();
                exit(EXIT_FAILURE);
            }

            break;

        case 'z':
            if (strcmp(optarg , "amp") == 0) {
                z_axis  = DISP_AMP;
            } else if (strcmp(optarg , "db") == 0) {
                z_axis  = DISP_db;
            } else if (strcmp(optarg , "phase") == 0) {
                z_axis  = DISP_PHASE;
            } else {
                help();
                exit(EXIT_FAILURE);
            }

            break;

        case 'i':
            strncpy(infile , optarg, strlen(optarg));
            break;

        case 'o':
            strncpy(outfile , optarg, strlen(optarg));
            break;

//        case 's':
//			samplingrate = atoi(optarg);
//	        break;

//        case 'n':
//			fft_len = atoi(optarg);
//	        break;

        case '?':
            help();
            exit(EXIT_FAILURE);
        }
    }


    // global_init();


    /*--------------------------------------------------*/
    /* 領域確保                                         */
    /*--------------------------------------------------*/
    // 伝達関数用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の領域確保
    //  g.Nsrc , g.Nmic , (g.Nfft/2+1)

    tf = harkio_TransferFunction_fromFile(infile, NULL);
    if (tf == NULL) {
        sprintf(msg,"cannot read zip file [%s]", infile);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }


    /*--------------------------------------------------*/
    //音源数
    /*--------------------------------------------------*/
    int num_src = tf->poses->size;


    /*---------------------------------------*/
    // nmic , nfft ([=512])
    /*---------------------------------------*/
    int samprate = tf->cfg->samplingRate;
    int num_mic = tf->mics->size;
    int num_fft = tf->cfg->nfft;





    /*-----------------------------------------------------------------------------*
    * ３次元配列を作成する(complex)
    * IN : size_x(Nsrc) x size_y(Nmic) x size_z( Nfft/2+1 )
    //g.TFvect[src][mic][fft]
    *-----------------------------------------------------------------------------*/
    // g.TFvect = harktool_newComplexThreeDim(  g.Nsrc , g.Nmic , g.Nfft );


    //伝達関数作業用３次元配列を作成する(complex)
    //*  Nsrc x Nmic x Nfft/2+1
    // 伝達関数 作業用領域
    // npos(72) x Nmic(8) x Nfft(512)/2 + 1 x sizeof(complex) 分の作業領域確保
    // harktool_new( TMP_TF_VECT );


    //伝達関数作成用TEMP領域
    // harktool_new( BUFFER );


    //結果格納用の行列
    harkio_Matrix** ifft_result = (harkio_Matrix**)malloc(num_fft);
    if (ifft_result == NULL) {
        harkio_Log_printf(harkio_Log_Error , "memory allocation error for ifft_result");
        exit(EXIT_FAILURE);
    }
    int src;
    for (src = 0; src < num_src ; src++) {
        ifft_result[src] = harkio_Matrix_new("complex", num_mic, num_fft);
        if (ifft_result[src] == NULL) {
            harkio_Log_printf(harkio_Log_Error , "memory error for ifft_result matrix");
            exit(EXIT_FAILURE);
        }
    }

    /*---------------------------------------*/
    // data get
    /*---------------------------------------*/
    if (tf_type == 0) {
        calcIFTMatrixData(tf->loctfs, ifft_result, num_src, num_mic, num_fft);
    } else { // tf_type == 1
        calcIFTMatrixData(tf->septfs, ifft_result, num_src, num_mic, num_fft);
    }



    /*---------------------------------------*/
    // file write
    /*---------------------------------------*/
    // writeCoordinate(  outfile , ifft_result , z_axis );
    float weight_fs = 1.0f/samprate;
    float weight_src = 0.2;
    writeCoordinateInternal(outfile, z_axis, ifft_result, num_src, num_mic, num_fft, weight_fs, weight_src);



    /*---------------------------------------*/
    // exit
    /*---------------------------------------*/
    // printf("SUCCESS\n");
    for (src=0; src<num_src; src++) {
        harkio_Matrix_delete(&ifft_result[src]);
    }
    free(ifft_result);
    // harktool_delete( BUFFER );

    return (EXIT_SUCCESS);

}




/*---------------------------------------*/
// Z軸計算
/*---------------------------------------*/
float calcZValue(int z_unit , float re , float im)
{

    char msg[MSG_LEN];


    switch (z_unit) {
    case DISP_REAL:
        return re;
        break;

    case DISP_AMP:
        return sqrtf(re*re + im*im);
        break;

    case DISP_db:
        return log10(sqrtf(re*re + im*im)) * 10;
        break;

    case DISP_PHASE:
        return atan2f(im, re) * 180/M_PI;
        break;

    default:
        sprintf(msg,"z_unit error [%d]", z_unit);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
        break;
    }

    return -1;
}




void calcIFTMatrixData(harkio_Matrix** freqdata, harkio_Matrix** timedata, int num_src, int num_mic, int num_fft)
{
    int cnt;
    int src;
    int mic;
    int fft;

    COMPLEX_FLOAT* buff_A;
    float* float_a;

    buff_A = (COMPLEX_FLOAT*)malloc((num_fft / 2 + 1) * sizeof(COMPLEX_FLOAT));
    if (buff_A == NULL) {
        harkio_Log_printf(harkio_Log_Error, "malloc error");
        exit(EXIT_FAILURE);
    }
    float_a = (float*)malloc(num_fft * sizeof(float));
    if (float_a == NULL) {
        harkio_Log_printf(harkio_Log_Error, "malloc error");
        exit(EXIT_FAILURE);
    }


    // iFFTの実行
    for (src = 0; src < num_src ; src++) { // f_max=g.Nsrc )
        for (mic = 0 ;  mic < num_mic ;  mic++) {
            for (fft = 0; fft < num_fft / 2 + 1 ; fft++) {
                cnt = fft + mic * (num_fft / 2 + 1);
                // buff_A[fft] =  g.TmpTFvect[src][mic][fft];
                buff_A[fft] = freqdata[src]->data.complexdata[cnt];
            }


            // fft実行 A[]の値をifftした結果がa[]にセットされる
            // Aのサイズ(複素対称)[0:Nfft/2] --> aのサイズ(実数のみ)[0:Nfft-1]
            // harktool_calcFloatIFFT( buff_A , float_a );
            harktool_FFTImpl_fRealFFTInv(buff_A, float_a, num_fft);

            //
            for (fft = 0; fft < num_fft ; fft++) {
                cnt = fft + mic * (num_fft);
                // g.TFvect[src][mic][fft] =  float_a[fft] + 0 * _Complex_I  ;
                timedata[src]->data.complexdata[cnt] = (COMPLEX_FLOAT)float_a[fft];
            }
        }
    }
    free(float_a);
}




/*------------------------------------------------------*/
//  座標変換
/*------------------------------------------------------*/
int writeCoordinateInternal(const char* outfile, int z_axis, harkio_Matrix** matdata, int num_src, int num_mic, int num_x, float x_weight, float y_weight)
{
    //printf("writeCoordinate start\n");

    FILE *fpw;
    float real;
    float imag;
    float z_data;
    char  msg[MSG_LEN];
    int   src;
    int   mic;
    int   x;


    if ((fpw = fopen(outfile  , "w")) == NULL) {
        sprintf(msg,"fopen error [%s]", outfile);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }

    //write header
    fprintf(fpw, "id,mic,x,y,z\n");

    for (src=0 ; src < num_src ; src++) {
        //x軸がHzの場合: 256個
        //y軸がsrcの場合: 72個(matファイル数)
        //z軸がAMPの場合:
        int cnt = 0;

        for (mic=0 ; mic < num_mic ; mic++) {
            for (x=0 ; x< num_x ; x++) {
                float x_data = x * x_weight ;   // TIME[s] or FREQ
                float y_data = src * y_weight ;

                //z axis
                real = crealf(matdata[src]->data.complexdata[cnt]);
                imag = cimagf(matdata[src]->data.complexdata[cnt]);

                z_data = calcZValue(z_axis , real  , imag);

                fprintf(fpw ,   "%d , %d , %f , %f , %f\n", src , mic , x_data , y_data , z_data);
                cnt++;
            }

        }//mic for
    }//src for


    fclose(fpw);

    return EXIT_SUCCESS;
}

