#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <libharkio3.h>



void help(void)
{
    printf("                                                                    \n");
    printf("                                                                    \n");
    printf("[NAME:]                                                             \n");
    printf("  harktoolcli-mergexmls                                             \n");
    printf("                                                                    \n");
    printf("[Description:]                                                      \n");
    printf("   merge multiple xml files                                         \n");
    printf("                                                                    \n");
    printf("[Usage :]                                                           \n");
    printf(" harktoolcli-mergexmls infile1 infile2 [optins]                     \n");
    printf("                                                                    \n");
    printf("[optios]                                                            \n");
    printf("      -o: output file name                 (-o out.xml)  [string]   \n");
    printf("		  - with no output file, write result into standard output  \n");
    printf("                                                                    \n");
    printf("                                                                    \n");
}



int main(int argc, char *argv[])
{

    harkio_XML* xmlsrc;
    harkio_XML* xmldst;

    int cnt;
    int cnt_in = 0;

    char* infiles[argc - 1]; //  using C99 feature (instead of malloc)
    char* outfile = NULL;
    FILE* fo = NULL;

    if (argc <= 1) {
    	harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help();
        exit(EXIT_FAILURE);
    }

    // help
    if (strcmp(argv[1] , "--help") == 0) {
        help();
        exit(EXIT_SUCCESS);
    }


    // get arguments
    for (cnt=1; cnt<argc; cnt++) {
        if (strcmp(argv[cnt] , "-o") == 0) {
            ++cnt;
            if (cnt >= argc) {
                harkio_Log_printf(harkio_Log_Error , "argument error!!");
                exit(EXIT_FAILURE);
            } else {
                outfile = argv[cnt];
            }
        } else {
            infiles[cnt_in] = argv[cnt];
            ++cnt_in;
        }
    }


    // read and assign xml data
    xmldst = harkio_XML_new();
    for (cnt = 0; cnt < cnt_in; cnt++) {
        xmlsrc = harkio_XML_newFromXML(infiles[cnt], "");

        // Positions
        if (xmlsrc->poses != NULL) {
            if (xmldst->poses != NULL) { ///"or if override"
                harkio_Log_printf(harkio_Log_Error , "<positions> Attribute duplicated!!");
                exit(EXIT_FAILURE);
            } else {
                xmldst->poses = harkio_Positions_copy(xmlsrc->poses);
            }
        }

        // Config
        if (xmlsrc->cfg != NULL) {
            if (xmldst->cfg != NULL) {
                harkio_Log_printf(harkio_Log_Error , "<config> Attribute duplicated!!");
                exit(EXIT_FAILURE);
            } else {
                xmldst->cfg = harkio_Config_copy(xmlsrc->cfg);
            }
        }

        // Neighbors
        if (xmlsrc->nbrs != NULL) {
            if (xmldst->nbrs != NULL) {
                harkio_Log_printf(harkio_Log_Error , "<neighbors> Attribute duplicated!!");
                exit(EXIT_FAILURE);
            } else {
                xmldst->nbrs = harkio_Neighbors_copy(xmlsrc->nbrs);
            }
        }

        harkio_XML_delete(&xmlsrc);
    }


    // set FILE* for output
    if (outfile != NULL) {
        if ((fo = fopen(outfile , "w")) == NULL) {
            harkio_Log_printf(harkio_Log_Error , "file open error!!");
            exit(EXIT_FAILURE);
        }
    } else {
        fo = stdout;
    }

    // write xml data
    harkio_XML_write(xmldst, fo);
    harkio_XML_delete(&xmlsrc);

    if (outfile != NULL) {
        fclose(fo);
    }
    return 0;
}

