#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <libharkio3.h>




#define ZDIR_TF          "transferFunction/"       //required
#define ZPATH_README     ZDIR_TF "README"          //optional
#define ZPATH_WHATISTHIS ZDIR_TF "whatisthis.txt"  //required
#define ZPATH_MIC        ZDIR_TF "microphones.xml" //required
#define ZPATH_SRC        ZDIR_TF "source.xml"      //required
#define ZDIR_TFLOC       ZDIR_TF "localization/"   //required
#define ZDIR_TFSEP       ZDIR_TF "separation/"     //required
#define ZPATH_TFLOC_PREF ZDIR_TFLOC "tf"           //either
#define ZPATH_TFSEP_PREF ZDIR_TFSEP "tf"           //either

#define ZFILENUM_REQUIRED  6
#define ZFILENUM_OPTIONAL  1







void help(void)
{
    printf("                                                \n");
    printf("                                                \n");
    printf("[NAME:]                                         \n");
    printf("  harktoolcli-validtf                           \n");
    printf("                                                \n");
    printf("[Description:]                                  \n");
    printf("  Consistency check of the transfer function    \n");
    printf("                                                \n");
    printf("[Usage :]                                       \n");
    printf("  harktoolcli-validtf tf_filename(zip)          \n");
    printf("                                                \n");
    printf("[Output :]                                      \n");
    printf("  If a tf_file is valid, the command retruns 0. \n");
    printf("                                                \n");
    printf("                                                \n");
}









int main(int argc, char **argv)
{

    int        errorp;
    int        index;
    const char *name;
    char *filename;
    char msg[MSG_LEN] = "";
    int  flags = 0;
    struct zip  *handle;
    zip_int64_t  znum;

    int posCnt;
    int posLocCheckCount;
    int posSepCheckCount;
    int cntZipFileRequired;
    int cntZipFileOptional;



    // argc check
    if (argc <= 1) {
        harkio_Log_printf(harkio_Log_Error , "argument error!!");
        help();
        exit(EXIT_FAILURE);
    }
    filename = argv[1];

    // help
    if (strcmp(argv[1] , "--help") == 0) {
        help();
        exit(EXIT_SUCCESS);
    }

    handle = zip_open(filename, ZIP_CHECKCONS, &errorp);

    if (handle == NULL) {
        sprintf(msg,"can not open zip file [%s]",filename);
        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    }

    //zip_int64_t zip_uint64_t zip_get_num_entries(struct zip *, int);
    znum = zip_get_num_entries(handle , flags);


    posLocCheckCount = 0;
    posSepCheckCount = 0 ;
    cntZipFileRequired = 0;
    cntZipFileOptional = 0;

    for (index=0 ; index< (int)znum ; index++) {
        name = zip_get_name(handle,index,flags);

        if (strcmp(name  , ZDIR_TF) == 0) {
            cntZipFileRequired++ ;
        } else if (strcmp(name  , ZPATH_README) == 0) {
            cntZipFileOptional++ ;
        } else if (strcmp(name  , ZPATH_WHATISTHIS) == 0) {
            cntZipFileRequired++ ;
        } else if (strcmp(name  , ZPATH_MIC) == 0) {
            cntZipFileRequired++ ;
        } else if (strcmp(name  , ZPATH_SRC) == 0) {
            cntZipFileRequired++ ;
        } else if (strcmp(name  , ZDIR_TFLOC) == 0) {
            cntZipFileRequired++ ;
        } else if (strcmp(name  , ZDIR_TFSEP) == 0) {
            cntZipFileRequired++ ;
        } else if (strncmp(name , ZPATH_TFLOC_PREF , strlen(ZPATH_TFLOC_PREF)) == 0) {
            posLocCheckCount++ ;
        } else if (strncmp(name , ZPATH_TFSEP_PREF , strlen(ZPATH_TFSEP_PREF)) == 0) {
            posSepCheckCount++ ;
        } else {
            printf("[Unknown] %d : %s\n", index, name);
            sprintf(msg,"File format error [%s] (zip count=%d)",filename , (int)znum);
            harkio_Log_printf(harkio_Log_Error , msg);
            zip_close(handle);
            handle = NULL;
            exit(EXIT_FAILURE);
        }
    }

    zip_close(handle);
    handle = NULL;




    //get position size
    harkio_TransferFunction *tf = NULL;

    tf = harkio_TransferFunction_fromFile(filename , NULL);
    if (tf == NULL) {
        harkio_Log_printf(harkio_Log_Error , "zip read file error");
        exit(EXIT_FAILURE);
    }
    posCnt = tf->poses->size ;

    harkio_TransferFunction_delete(&tf);




    int check = 1; //true

    check = check && (cntZipFileRequired == ZFILENUM_REQUIRED);
    check = check && (
                (posLocCheckCount == posCnt && posSepCheckCount == 0) ||
                (posLocCheckCount == 0        && posSepCheckCount == posCnt) ||
                (posLocCheckCount == posCnt && posSepCheckCount == posCnt));

    if (check == 0) {
        sprintf(msg,"File format error [%s] (zip count=%d, localization file=%d, separation file=%d)"
                ,filename
                , (int)znum
                , posLocCheckCount
                ,posSepCheckCount);

        harkio_Log_printf(harkio_Log_Error , msg);
        exit(EXIT_FAILURE);
    } else {
        fprintf(stderr, "SUCCESS\n"); // "Successfully validated."
    }

    return (EXIT_SUCCESS);

}




